@@ -85,6 +85,7 @@
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <openssl/x509.h>
+#include <openssl/x509v3.h>
 #include <openssl/pem.h>
 #include <openssl/crypto.h>
 #include <openssl/evp.h>
@@ -679,8 +680,8 @@ void          ssl_ds_table_kill(ssl_ds_table *);
 /*  Mutex Support  */
 void         ssl_mutex_init(server_rec *, pool *);
 void         ssl_mutex_open(server_rec *, pool *);
-void         ssl_mutex_on(void);
-void         ssl_mutex_off(void);
+void         ssl_mutex_on(server_rec *);
+void         ssl_mutex_off(server_rec *);
 void         ssl_mutex_file_create(server_rec *, pool *);
 void         ssl_mutex_file_open(server_rec *, pool *);
 void         ssl_mutex_file_remove(void *);
