@@ -764,6 +764,8 @@ const char *ssl_cmd_SSLOptions(
             opt = SSL_OPT_FAKEBASICAUTH;
         else if (strcEQ(w, "StrictRequire"))
             opt = SSL_OPT_STRICTREQUIRE;
+        else if (strcEQ(w, "OptRenegotiate"))
+            opt = SSL_OPT_OPTRENEGOTIATE;
         else
             return ap_pstrcat(cmd->pool, "SSLOptions: Illegal option '", w, "'", NULL);
 
