@@ -134,6 +134,7 @@ void ssl_hook_NewConnection(conn_rec *conn)
     SSL *ssl;
     char *cp;
     char *cpVHostID;
+    X509 *xs;
     int rc;
     int n;
 
@@ -396,18 +397,12 @@ void ssl_hook_NewConnection(conn_rec *conn)
         }
 
         /*
-         * Remember the peer certificate when
-         * client authentication was done
+         * Remember the peer certificate's DN
          */
-        if (sc->nVerifyClient != SSL_CVERIFY_NONE) {
-            char *s;
-            X509 *xs;
-
-            if ((xs = SSL_get_peer_certificate(ssl)) != NULL) {
-                s = X509_NAME_oneline(X509_get_subject_name(xs), NULL, 0);
-                ap_ctx_set(fb->ctx, "ssl::client::dn", ap_pstrdup(conn->pool, s));
-                free(s);
-            }
+        if ((xs = SSL_get_peer_certificate(ssl)) != NULL) {
+            cp = X509_NAME_oneline(X509_get_subject_name(xs), NULL, 0);
+            ap_ctx_set(fb->ctx, "ssl::client::dn", ap_pstrdup(conn->pool, cp));
+            free(cp);
         }
 
         /*
@@ -1000,6 +995,16 @@ int ssl_hook_Access(request_rec *r)
             }
         }
 
+        /*
+         * Remember the peer certificate's DN
+         */
+        if ((cert = SSL_get_peer_certificate(ssl)) != NULL) {
+            cp = X509_NAME_oneline(X509_get_subject_name(cert), NULL, 0);
+            ap_ctx_set(r->connection->client->ctx, "ssl::client::dn", 
+                       ap_pstrdup(r->connection->pool, cp));
+            free(cp);
+        }
+
         /*
          * Finally check for acceptable renegotiation results
          */
@@ -1105,7 +1110,7 @@ int ssl_hook_Auth(request_rec *r)
         return FORBIDDEN;
 
     /*
-     * We decline operation in various situations..
+     * We decline operation in various situations...
      */
     if (!sc->bEnabled)
         return DECLINED;
