@@ -97,25 +97,31 @@ void ssl_mutex_open(server_rec *s, pool *p)
     return;
 }
 
-void ssl_mutex_on(void)
+void ssl_mutex_on(server_rec *s)
 {
     SSLModConfigRec *mc = myModConfig();
+    BOOL ok = TRUE;
 
     if (mc->nMutexMode == SSL_MUTEXMODE_FILE)
-        ssl_mutex_file_acquire();
+        ok = ssl_mutex_file_acquire();
     else if (mc->nMutexMode == SSL_MUTEXMODE_SEM)
-        ssl_mutex_sem_acquire();
+        ok = ssl_mutex_sem_acquire();
+    if (!ok)
+        ssl_log(s, SSL_LOG_WARN, "Failed to acquire global mutex lock");
     return;
 }
 
-void ssl_mutex_off(void)
+void ssl_mutex_off(server_rec *s)
 {
     SSLModConfigRec *mc = myModConfig();
+    BOOL ok = TRUE;
 
     if (mc->nMutexMode == SSL_MUTEXMODE_FILE)
-        ssl_mutex_file_release();
+        ok = ssl_mutex_file_release();
     else if (mc->nMutexMode == SSL_MUTEXMODE_SEM)
-        ssl_mutex_sem_release();
+        ok = ssl_mutex_sem_release();
+    if (!ok)
+        ssl_log(s, SSL_LOG_WARN, "Failed to release global mutex lock");
     return;
 }
 
@@ -326,7 +332,7 @@ BOOL ssl_mutex_sem_release(void)
 
 #ifdef SSL_HAVE_IPCSEM
     struct sembuf sb[] = {
-        { 0, -1, SEM_UNDO } /* derements semaphore */
+        { 0, -1, SEM_UNDO } /* decrements semaphore */
     };
 
     rc = semop(mc->nMutexSEMID, sb, 1);
