@@ -122,6 +122,9 @@ void ssl_scache_store(server_rec *s, SSL_SESSION *pSession, int timeout)
     ap_hook_use("ap::mod_ssl::vendor::scache_store",
                 AP_HOOK_SIG3(void,ptr,ptr), AP_HOOK_ALL, s, &SCI);
 #endif
+    
+    /* allow the regular expiring to occur */
+    ssl_scache_expire(s, time(NULL));
 
     return;
 }
@@ -234,6 +237,9 @@ void ssl_scache_status(server_rec *s, pool *p, void (*func)(char *, void *), voi
 {
     SSLModConfigRec *mc = myModConfig();
 
+    /* allow the regular expiring to occur */
+    ssl_scache_expire(s, time(NULL));
+
     if (mc->nSessionCacheMode == SSL_SCMODE_DBM)
         ssl_scache_dbm_status(s, p, func, arg);
     else if (mc->nSessionCacheMode == SSL_SCMODE_SHM)
@@ -282,13 +288,13 @@ void ssl_scache_dbm_init(server_rec *s, pool *p)
      * Open it once to create it and to make sure it
      * _can_ be created.
      */
-    ssl_mutex_on();
+    ssl_mutex_on(s);
     if ((dbm = ssl_dbm_open(mc->szSessionCacheDataFile,
                             O_RDWR|O_CREAT, SSL_DBM_FILE_MODE)) == NULL) {
         ssl_log(s, SSL_LOG_ERROR|SSL_ADD_ERRNO,
                 "Cannot create SSLSessionCache DBM file `%s'",
                 mc->szSessionCacheDataFile);
-        ssl_mutex_off();
+        ssl_mutex_off(s);
         return;
     }
     ssl_dbm_close(dbm);
@@ -309,7 +315,7 @@ void ssl_scache_dbm_init(server_rec *s, pool *p)
               ap_user_id, -1 /* no gid change */);
     }
 #endif
-    ssl_mutex_off();
+    ssl_mutex_off(s);
 
     return;
 }
@@ -334,18 +340,18 @@ void ssl_scache_dbm_store(server_rec *s, ssl_scinfo_t *SCI)
     memcpy((char *)dbmval.dptr+sizeof(time_t), SCI->ucaData, SCI->nData);
 
     /* and store it to the DBM file */
-    ssl_mutex_on();
+    ssl_mutex_on(s);
     if ((dbm = ssl_dbm_open(mc->szSessionCacheDataFile,
                             O_RDWR, SSL_DBM_FILE_MODE)) == NULL) {
         ssl_log(s, SSL_LOG_ERROR|SSL_ADD_ERRNO,
                 "Cannot open SSLSessionCache DBM file `%s' for writing (store)",
                 mc->szSessionCacheDataFile);
-        ssl_mutex_off();
+        ssl_mutex_off(s);
         return;
     }
     ssl_dbm_store(dbm, dbmkey, dbmval, DBM_INSERT);
     ssl_dbm_close(dbm);
-    ssl_mutex_off();
+    ssl_mutex_off(s);
 
     /* free temporary buffers */
     free(dbmval.dptr);
@@ -370,18 +376,18 @@ void ssl_scache_dbm_retrieve(server_rec *s, ssl_scinfo_t *SCI)
     dbmkey.dsize = SCI->nKey;
 
     /* and fetch it from the DBM file */
-    ssl_mutex_on();
+    ssl_mutex_on(s);
     if ((dbm = ssl_dbm_open(mc->szSessionCacheDataFile,
                             O_RDONLY, SSL_DBM_FILE_MODE)) == NULL) {
         ssl_log(s, SSL_LOG_ERROR|SSL_ADD_ERRNO,
                 "Cannot open SSLSessionCache DBM file `%s' for reading (fetch)",
                 mc->szSessionCacheDataFile);
-        ssl_mutex_off();
+        ssl_mutex_off(s);
         return;
     }
     dbmval = ssl_dbm_fetch(dbm, dbmkey);
     ssl_dbm_close(dbm);
-    ssl_mutex_off();
+    ssl_mutex_off(s);
 
     /* immediately return if not found */
     if (dbmval.dptr == NULL || dbmval.dsize < sizeof(time_t))
@@ -411,18 +417,18 @@ void ssl_scache_dbm_remove(server_rec *s, ssl_scinfo_t *SCI)
     dbmkey.dsize = SCI->nKey;
 
     /* and delete it from the DBM file */
-    ssl_mutex_on();
+    ssl_mutex_on(s);
     if ((dbm = ssl_dbm_open(mc->szSessionCacheDataFile,
                             O_RDWR, SSL_DBM_FILE_MODE)) == NULL) {
         ssl_log(s, SSL_LOG_ERROR|SSL_ADD_ERRNO,
                 "Cannot open SSLSessionCache DBM file `%s' for writing (delete)",
                 mc->szSessionCacheDataFile);
-        ssl_mutex_off();
+        ssl_mutex_off(s);
         return;
     }
     ssl_dbm_delete(dbm, dbmkey);
     ssl_dbm_close(dbm);
-    ssl_mutex_off();
+    ssl_mutex_off(s);
 
     return;
 }
@@ -432,34 +438,43 @@ void ssl_scache_dbm_expire(server_rec *s, time_t tNow)
     SSLModConfigRec *mc = myModConfig();
     DBM *dbm;
     datum dbmkey;
+    datum dbmkeythis;
     datum dbmval;
     time_t tExpiresAt;
+    int nElements = 0;
+    int bDelete;
 
     ssl_log(s, SSL_LOG_TRACE, "Expiring DBM-based SSL session cache");
-    ssl_mutex_on();
+    ssl_mutex_on(s);
     if ((dbm = ssl_dbm_open(mc->szSessionCacheDataFile,
                             O_RDWR, SSL_DBM_FILE_MODE)) == NULL) {
         ssl_log(s, SSL_LOG_ERROR|SSL_ADD_ERRNO,
                 "Cannot open SSLSessionCache DBM file `%s' for expiring",
                 mc->szSessionCacheDataFile);
-        ssl_mutex_off();
+        ssl_mutex_off(s);
         return;
     }
     dbmkey = ssl_dbm_firstkey(dbm);
-    for ( ; dbmkey.dptr != NULL; dbmkey = ssl_dbm_nextkey(dbm)) {
-        dbmval = ssl_dbm_fetch(dbm, dbmkey);
-        if (dbmval.dptr == NULL)
-            continue;
-        if (dbmval.dsize < sizeof(time_t)) {
-            ssl_dbm_delete(dbm, dbmkey);
-            continue;
-        }
-        memcpy(&tExpiresAt, dbmval.dptr, sizeof(time_t));
-        if (tExpiresAt >= tNow)
-            ssl_dbm_delete(dbm, dbmkey);
+    if (dbmkey.dptr != NULL) {
+        do {
+            bDelete = FALSE;
+            dbmval = ssl_dbm_fetch(dbm, dbmkey);
+            if (dbmval.dsize < sizeof(time_t) || dbmval.dptr == NULL)
+                bDelete = TRUE;
+            else {
+                memcpy(&tExpiresAt, dbmval.dptr, sizeof(time_t));
+                if (tExpiresAt >= tNow)
+                    bDelete = TRUE;
+            }
+            dbmkeythis = dbmkey;
+            dbmkey = ssl_dbm_nextkey(dbm);
+            if (bDelete)
+                ssl_dbm_delete(dbm, dbmkeythis);
+        } while (dbmkey.dptr != NULL);
     }
     ssl_dbm_close(dbm);
-    ssl_mutex_off();
+    ssl_mutex_off(s);
+    ssl_log(s, SSL_LOG_INFO, "Session Cache Status: %d elements", nElements);
     return;
 }
 
@@ -475,13 +490,13 @@ void ssl_scache_dbm_status(server_rec *s, pool *p, void (*func)(char *, void *),
 
     nElem = 0;
     nSize = 0;
-    ssl_mutex_on();
+    ssl_mutex_on(s);
     if ((dbm = ssl_dbm_open(mc->szSessionCacheDataFile,
                             O_RDONLY, SSL_DBM_FILE_MODE)) == NULL) {
         ssl_log(s, SSL_LOG_ERROR|SSL_ADD_ERRNO,
                 "Cannot open SSLSessionCache DBM file `%s' for status retrival",
                 mc->szSessionCacheDataFile);
-        ssl_mutex_off();
+        ssl_mutex_off(s);
         return;
     }
     dbmkey = ssl_dbm_firstkey(dbm);
@@ -493,7 +508,7 @@ void ssl_scache_dbm_status(server_rec *s, pool *p, void (*func)(char *, void *),
         nSize += dbmval.dsize;
     }
     ssl_dbm_close(dbm);
-    ssl_mutex_off();
+    ssl_mutex_off(s);
     if (nSize > 0 && nElem > 0)
         nAverage = nSize / nElem;
     else
@@ -609,17 +624,17 @@ void ssl_scache_shm_store(server_rec *s, ssl_scinfo_t *SCI)
     SSLModConfigRec *mc = myModConfig();
     void *vp;
 
-    ssl_mutex_on();
+    ssl_mutex_on(s);
     if (table_insert_kd(mc->tSessionCacheDataTable, 
                         SCI->ucaKey, SCI->nKey, 
                         NULL, sizeof(time_t)+SCI->nData,
                         NULL, &vp, 1) != TABLE_ERROR_NONE) {
-        ssl_mutex_off();
+        ssl_mutex_off(s);
         return;
     }
     memcpy(vp, &SCI->tExpiresAt, sizeof(time_t));
     memcpy((char *)vp+sizeof(time_t), SCI->ucaData, SCI->nData);
-    ssl_mutex_off();
+    ssl_mutex_off(s);
     return;
 }
 
@@ -635,11 +650,11 @@ void ssl_scache_shm_retrieve(server_rec *s, ssl_scinfo_t *SCI)
     SCI->tExpiresAt = 0;
 
     /* lookup key in table */
-    ssl_mutex_on();
+    ssl_mutex_on(s);
     if (table_retrieve(mc->tSessionCacheDataTable,
                        SCI->ucaKey, SCI->nKey, 
                        &vp, &n) != TABLE_ERROR_NONE) {
-        ssl_mutex_off();
+        ssl_mutex_off(s);
         return;
     }
 
@@ -648,12 +663,12 @@ void ssl_scache_shm_retrieve(server_rec *s, ssl_scinfo_t *SCI)
     SCI->ucaData = (UCHAR *)malloc(SCI->nData);
     if (SCI->ucaData == NULL) {
         SCI->nData = 0;
-        ssl_mutex_off();
+        ssl_mutex_off(s);
         return;
     }
     memcpy(&SCI->tExpiresAt, vp, sizeof(time_t));
     memcpy(SCI->ucaData, (char *)vp+sizeof(time_t), SCI->nData);
-    ssl_mutex_off();
+    ssl_mutex_off(s);
 
     return;
 }
@@ -663,45 +678,56 @@ void ssl_scache_shm_remove(server_rec *s, ssl_scinfo_t *SCI)
     SSLModConfigRec *mc = myModConfig();
 
     /* remove value under key in table */
-    ssl_mutex_on();
+    ssl_mutex_on(s);
     table_delete(mc->tSessionCacheDataTable,
                  SCI->ucaKey, SCI->nKey, NULL, NULL);
-    ssl_mutex_off();
+    ssl_mutex_off(s);
     return;
 }
 
 void ssl_scache_shm_expire(server_rec *s, time_t tNow)
 {
     SSLModConfigRec *mc = myModConfig();
+    table_linear_t iterator;
     time_t tExpiresAt;
     void *vpKey;
+    void *vpKeyThis;
     void *vpData;
     int nKey;
+    int nKeyThis;
     int nData;
     int nElements = 0;
+    int nDeleted = 0;
+    int bDelete;
+    int rc;
 
-    ssl_log(s, SSL_LOG_TRACE, "Expiring SHM-based SSL session cache");
-    ssl_mutex_on();
-    if (table_first(mc->tSessionCacheDataTable,
-                    &vpKey, &nKey, &vpData, &nData) == TABLE_ERROR_NONE) {
+    ssl_mutex_on(s);
+    if (table_first_r(mc->tSessionCacheDataTable, &iterator,
+                      &vpKey, &nKey, &vpData, &nData) == TABLE_ERROR_NONE) {
         do {
+            bDelete = FALSE;
             nElements++;
-            if (vpKey == NULL || vpData == NULL)
-                continue;
-            if (nData < sizeof(time_t)) {
-                table_delete(mc->tSessionCacheDataTable,
-                             vpKey, nKey, NULL, NULL);
-                continue;
+            if (nData < sizeof(time_t) || vpData == NULL)
+                bDelete = TRUE;
+            else {
+                memcpy(&tExpiresAt, vpData, sizeof(time_t));
+                if (tExpiresAt >= tNow)
+                   bDelete = TRUE;
             }
-            memcpy(&tExpiresAt, vpData, sizeof(time_t));
-            if (tExpiresAt >= tNow)
+            vpKeyThis = vpKey;
+            nKeyThis  = nKey;
+            rc = table_next_r(mc->tSessionCacheDataTable, &iterator,
+                              &vpKey, &nKey, &vpData, &nData);
+            if (bDelete) {
                 table_delete(mc->tSessionCacheDataTable,
-                             vpKey, nKey, NULL, NULL);
-        } while (table_next(mc->tSessionCacheDataTable,
-                            &vpKey, &nKey, &vpData, &nData) == TABLE_ERROR_NONE);
+                             vpKeyThis, nKeyThis, NULL, NULL);
+                nDeleted++;
+            }
+        } while (rc == TABLE_ERROR_NONE);
     }
-    ssl_mutex_off();
-    ssl_log(s, SSL_LOG_INFO, "Session Cache Status: %d elements", nElements);
+    ssl_mutex_off(s);
+    ssl_log(s, SSL_LOG_INFO, "Expired SHM-based SSL session cache: "
+            "old: %d, new: %d, removed: %d", nElements+nDeleted, nElements, nDeleted);
     return;
 }
 
@@ -718,7 +744,7 @@ void ssl_scache_shm_status(server_rec *s, pool *p, void (*func)(char *, void *),
 
     nElem = 0;
     nSize = 0;
-    ssl_mutex_on();
+    ssl_mutex_on(s);
     if (table_first(mc->tSessionCacheDataTable,
                     &vpKey, &nKey, &vpData, &nData) == TABLE_ERROR_NONE) {
         do {
@@ -729,7 +755,7 @@ void ssl_scache_shm_status(server_rec *s, pool *p, void (*func)(char *, void *),
         } while (table_next(mc->tSessionCacheDataTable,
                             &vpKey, &nKey, &vpData, &nData) == TABLE_ERROR_NONE);
     }
-    ssl_mutex_off();
+    ssl_mutex_off(s);
     if (nSize > 0 && nElem > 0)
         nAverage = nSize / nElem;
     else
